function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _object_spread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _define_property(target, key, source[key]);
        });
    }
    return target;
}
function _object_without_properties(source, excluded) {
    if (source == null) return {};
    var target = _object_without_properties_loose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for(i = 0; i < sourceSymbolKeys.length; i++){
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) continue;
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
            target[key] = source[key];
        }
    }
    return target;
}
function _object_without_properties_loose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for(i = 0; i < sourceKeys.length; i++){
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        target[key] = source[key];
    }
    return target;
}
import { AxiosError } from 'axios';
import { buildFullPath, deepCopy, generateRequestHash } from '../utils';
import { addCanceller, deleteCalledCancellerByHash, deleteCancellerByHash, hasCalledCancelerRecord } from '../canceller';
import { addRequestCanceler, clearCancelerByHash } from '../cancelLastSameRequest';
export default function wxAdapter(config) {
    // adapter接受的config要完全符合axios的配置规范，然后在这里针对不同的引擎做个性化修改
    return new Promise(function(resolve, reject) {
        var requestConfig = {};
        var isAborted = false;
        var requestTask = null;
        var headers = config.headers, data = config.data, params = config.params, url = config.url, timeout = config.timeout, method = config.method, rest = _object_without_properties(config, [
            "headers",
            "data",
            "params",
            "url",
            "timeout",
            "method"
        ]);
        var requestHash = generateRequestHash(config);
        var requestHeader = deepCopy(headers);
        delete requestHeader.Referer; // 小程序文档，header中不能有Referer
        requestConfig.header = requestHeader;
        var requestData = data || params; // wx环境，不论get还是post，参数传递的属性名均为data
        requestConfig.data = deepCopy(requestData);
        requestConfig.url = buildFullPath(config.baseURL || '', url || '');
        requestConfig.timeout = timeout;
        requestConfig.method = method === null || method === void 0 ? void 0 : method.toUpperCase();
        requestConfig = _object_spread({}, requestConfig, rest);
        requestConfig.success = function(res) {
            var response = {};
            var resData = res.data, statusCode = res.statusCode, header = res.header, restRes = _object_without_properties(res, [
                "data",
                "statusCode",
                "header"
            ]);
            response.data = resData;
            response.status = statusCode;
            response.headers = header;
            response.config = config;
            response = _object_spread({}, response, restRes);
            var validateStatus = response.config.validateStatus;
            if (!response.status || !validateStatus || validateStatus(response.status)) {
                resolve(response);
            } else {
                var errMsg = "Request failed with status code ".concat(response.status);
                reject(new AxiosError(errMsg, '', config, requestTask, response));
            }
        };
        requestConfig.fail = function(err) {
            if (isAborted) {
                reject(new AxiosError(err.errMsg, AxiosError.ERR_CANCELED, config, requestTask));
            }
            reject(new AxiosError(err.errMsg, err.errno, config, requestTask));
        };
        // 判断是否已经调用了取消函数
        if (hasCalledCancelerRecord(requestConfig.cancelKey, requestHash)) {
            // 已经调用过取消函数了，直接抛出错误即可
            reject(new AxiosError('请求在发出之前被主动取消！', AxiosError.ERR_CANCELED, config, requestTask));
            // 删除记录
            deleteCalledCancellerByHash(requestConfig.cancelKey, requestHash);
            return;
        }
        requestTask = wx.request(requestConfig);
        // 手动取消请求功能
        // 设置了cancelKey，但是环境不支持AbortController
        if ('undefined' === typeof AbortController) {
            if (requestConfig.cancelKey) {
                addCanceller(requestConfig.cancelKey, requestHash, function() {
                    isAborted = true;
                    requestTask.abort.call(requestTask);
                    deleteCancellerByHash(requestConfig.cancelKey, requestHash);
                    reject(new AxiosError('请求被取消！', AxiosError.ERR_CANCELED, config, requestTask));
                });
            }
            // 取消上一个相同URL的get请求功能
            if (requestConfig.cancelLastSameUrl && requestConfig.method.toLocaleLowerCase() === 'get') {
                addRequestCanceler(config.url, requestHash, function() {
                    isAborted = true;
                    requestTask.abort.call(requestTask);
                    clearCancelerByHash(config.url, requestHash);
                    reject(new AxiosError('请求被取消！', AxiosError.ERR_CANCELED, config, requestTask));
                });
            }
        }
    });
}
wxAdapter.type = 'wx';
