function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _object_spread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _define_property(target, key, source[key]);
        });
    }
    return target;
}
function _object_without_properties(source, excluded) {
    if (source == null) return {};
    var target = _object_without_properties_loose(source, excluded);
    var key, i;
    if (Object.getOwnPropertySymbols) {
        var sourceSymbolKeys = Object.getOwnPropertySymbols(source);
        for(i = 0; i < sourceSymbolKeys.length; i++){
            key = sourceSymbolKeys[i];
            if (excluded.indexOf(key) >= 0) continue;
            if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
            target[key] = source[key];
        }
    }
    return target;
}
function _object_without_properties_loose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for(i = 0; i < sourceKeys.length; i++){
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        target[key] = source[key];
    }
    return target;
}
function _type_of(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
}
import qs from 'qs';
import { callCancellerByCancelKey, hasCancelers } from './canceller';
/**
 * 标准化config配置，或根据配置项设置相关的参数
 * 该函数返回的配置对象，应该完全符合axios的配置规范
 * @param {*} config
 */ export function normalizeConfig(ctx, config) {
    var _ref = _object_spread({}, ctx.$options, config), _ref_headers = _ref.headers, headers = _ref_headers === void 0 ? {} : _ref_headers, _ref_sendFormData = _ref.sendFormData, sendFormData = _ref_sendFormData === void 0 ? false : _ref_sendFormData, _ref_data = _ref.data, data = _ref_data === void 0 ? undefined : _ref_data, _ref_params = _ref.params, params = _ref_params === void 0 ? {} : _ref_params, _ref_baseUrl = _ref.baseUrl, baseUrl = _ref_baseUrl === void 0 ? '' : _ref_baseUrl, _ref_baseURL = _ref.baseURL, baseURL = _ref_baseURL === void 0 ? '' : _ref_baseURL, _ref_reUseDuration = _ref.reUseDuration, reUseDuration = _ref_reUseDuration === void 0 ? 300 : _ref_reUseDuration, rest = _object_without_properties(_ref, [
        "headers",
        "sendFormData",
        "data",
        "params",
        "baseUrl",
        "baseURL",
        "reUseDuration"
    ]);
    // 数据格式为FormData，修改content-type头，以及修改data格式
    var contentType = headers['content-type'] || headers['Content-Type'] || 'application/json';
    headers['content-type'] = sendFormData ? 'application/x-www-form-urlencoded' : contentType;
    var sendData = sendFormData && (typeof data === "undefined" ? "undefined" : _type_of(data)) === 'object' && data !== null ? qs.stringify(data) : data;
    // baseUrl，支持两种写法，且用户配置的baseUrl优先级最高
    var bUrl = baseUrl || baseURL || ctx.$baseUrl;
    if (typeof reUseDuration !== 'number' || Number.isNaN(reUseDuration)) {
        throw new Error('ajax配置reUseDuration必须是数字');
    }
    return _object_spread({
        headers: headers,
        params: params,
        data: sendData,
        baseURL: bUrl,
        reUseDuration: reUseDuration
    }, rest);
}
export function addInstanceExtendMethod(ctx) {
    var extend = function(source) {
        Object.keys(source).forEach(function(key) {
            ctx[key] = source[key];
        });
    };
    ctx.extend = extend;
}
export function proxyAxiosInstanceAttrs(target, source) {
    var attrs = [
        'request',
        'interceptors',
        'defaults'
    ];
    // const methods: ProxyedAttrs[] = ['request', 'get', 'post', 'delete', 'head', 'options', 'put', 'patch', 'interceptors'];
    attrs.forEach(function(method) {
        Object.defineProperty(target, method, {
            value: source[method],
            writable: false
        });
    });
}
export function proxyAjaxCanceler(ctx) {
    ctx.canceller = new Proxy({}, {
        get: function get(target, prop) {
            if (!hasCancelers(prop)) {
                return function() {
                    var str = "ajax.canceller对象上不存在名为".concat(prop, "的函数！");
                    console.warn(str);
                    return str;
                };
            }
            return function() {
                callCancellerByCancelKey(prop);
            };
        }
    });
}
