function _define_property(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _object_spread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _define_property(target, key, source[key]);
        });
    }
    return target;
}
function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);
    if (Object.getOwnPropertySymbols) {
        var symbols = Object.getOwnPropertySymbols(object);
        if (enumerableOnly) {
            symbols = symbols.filter(function(sym) {
                return Object.getOwnPropertyDescriptor(object, sym).enumerable;
            });
        }
        keys.push.apply(keys, symbols);
    }
    return keys;
}
function _object_spread_props(target, source) {
    source = source != null ? source : {};
    if (Object.getOwnPropertyDescriptors) {
        Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
        ownKeys(Object(source)).forEach(function(key) {
            Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
        });
    }
    return target;
}
function _type_of(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
}
import hash from 'object-hash';
export function isAbsoluteURL(url) {
    // A URL is considered absolute if it begins with "<scheme>://" or "//" (protocol-relative URL).
    // RFC 3986 defines scheme name as a sequence of characters beginning with a letter and followed
    // by any combination of letters, digits, plus, period, or hyphen.
    return /^([a-z][a-z\d+-.]*:)?\/\//i.test(url);
}
export function combineURLs(baseURL, relativeURL) {
    return relativeURL ? "".concat(baseURL.replace(/\/+$/, ''), "/").concat(relativeURL.replace(/^\/+/, '')) : baseURL;
}
export function buildFullPath(baseURL, requestedURL) {
    if (baseURL && !isAbsoluteURL(requestedURL)) {
        return combineURLs(baseURL, requestedURL);
    }
    return requestedURL;
}
export function deepCopy(obj) {
    var target;
    if ((typeof obj === "undefined" ? "undefined" : _type_of(obj)) === 'object' && obj !== null) {
        if (Array.isArray(obj)) {
            target = [];
            obj.forEach(function(elem) {
                return target.push(deepCopy(elem));
            });
        } else {
            target = {};
            Object.keys(obj).forEach(function(key) {
                target[key] = deepCopy(obj[key]);
            });
        }
    } else {
        target = obj;
    }
    return target;
}
export function getType(obj) {
    var type = typeof obj === "undefined" ? "undefined" : _type_of(obj);
    if (type !== 'object') return type;
    return Object.prototype.toString.call(obj).replace(/^\[object\s+(\S+)\]$/, '$1').toLowerCase();
}
/**
 * 获取请求的标识，相同的请求配置有相同的标识
 * @param config 经过标准化的ajax请求配置
 * @returns 该请求的hash key
 */ export function generateRequestHash(config) {
    var _config_url = config.url, url = _config_url === void 0 ? '' : _config_url, _config_method = config.method, method = _config_method === void 0 ? '' : _config_method, _config_params = config.params, params = _config_params === void 0 ? {} : _config_params, _config_data = config.data, data = _config_data === void 0 ? {} : _config_data, _config_baseURL = config.baseURL, baseURL = _config_baseURL === void 0 ? '' : _config_baseURL;
    return hash(_object_spread_props(_object_spread({}, params, data), {
        baseURL: baseURL,
        url: url,
        method: method
    }));
}
